/*
 * SPDX-FileCopyrightText: 2014 Hugo Pereira Da Costa <hugo.pereira@free.fr>
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

#include "breezepropertynames.h"

namespace Breeze
{

    const char PropertyNames::noAnimations[] = "_kde_no_animations";
    const char PropertyNames::noWindowGrab[] = "_kde_no_window_grab";
    const char PropertyNames::netWMForceShadow[] = "_KDE_NET_WM_FORCE_SHADOW";
    const char PropertyNames::netWMSkipShadow[] = "_KDE_NET_WM_SKIP_SHADOW";
    const char PropertyNames::sidePanelView[] = "_kde_side_panel_view";
    const char PropertyNames::toolButtonAlignment[] = "_kde_toolButton_alignment";
    const char PropertyNames::menuTitle[] = "_breeze_toolButton_menutitle";
    const char PropertyNames::alteredBackground[] = "_breeze_altered_background";
    const char PropertyNames::highlightNeutral[] = "_kde_highlight_neutral";

}
